@@ -1,5 +1,5 @@
 
-import random
+import random, weakref
 from zope.interface import implements
 from twisted.python import log, components
 from twisted.spread import pb
@@ -253,6 +253,8 @@ class Builder(pb.Referenceable):
     @type building: list of L{buildbot.process.base.Build}
     @ivar building: Builds that are actively running
 
+    @type slaves: list of L{buildbot.buildslave.BuildSlave} objects
+    @ivar slaves: the slaves currently available for building
     """
 
     expectations = None # this is created the first time we get a good build
@@ -284,6 +286,8 @@ class Builder(pb.Referenceable):
         # build/wannabuild slots: Build objects move along this sequence
         self.buildable = []
         self.building = []
+        # old_building holds active builds that were stolen from a predecessor
+        self.old_building = weakref.WeakKeyDictionary()
 
         # buildslaves which have connected but which are not yet available.
         # These are always in the ATTACHING state.
@@ -376,13 +380,21 @@ class Builder(pb.Referenceable):
         self.buildable.extend(old.buildable)
         old.buildable = []
 
-        # old.building is not migrated: it keeps track of builds which were
-        # in progress in the old Builder. When those builds finish, the old
-        # Builder will be notified, not us. However, since the old
-        # SlaveBuilder will point to us, it is our maybeStartBuild() that
-        # will be triggered.
+        # old.building (i.e. builds which are still running) is not migrated
+        # directly: it keeps track of builds which were in progress in the
+        # old Builder. When those builds finish, the old Builder will be
+        # notified, not us. However, since the old SlaveBuilder will point to
+        # us, it is our maybeStartBuild() that will be triggered.
         if old.building:
             self.builder_status.setBigState("building")
+        # however, we do grab a weakref to the active builds, so that our
+        # BuilderControl can see them and stop them. We use a weakref because
+        # we aren't the one to get notified, so there isn't a convenient
+        # place to remove it from self.building .
+        for b in old.building:
+            self.old_building[b] = None
+        for b in old.old_building:
+            self.old_building[b] = None
 
         # Our set of slavenames may be different. Steal any of the old
         # buildslaves that we want to keep using.
@@ -414,6 +426,15 @@ class Builder(pb.Referenceable):
 
         return # all done
 
+    def getBuild(self, number):
+        for b in self.building:
+            if b.build_status.number == number:
+                return b
+        for b in self.old_building.keys():
+            if b.build_status.number == number:
+                return b
+        return None
+
     def fireTestEvent(self, name, fire_with=None):
         if fire_with is None:
             fire_with = self
@@ -426,7 +447,7 @@ class Builder(pb.Referenceable):
         """This is invoked by the BuildSlave when the self.slavename bot
         registers their builder.
 
-        @type  slave: L{buildbot.master.BuildSlave}
+        @type  slave: L{buildbot.buildslave.BuildSlave}
         @param slave: the BuildSlave that represents the buildslave as a whole
         @type  remote: L{twisted.spread.pb.RemoteReference}
         @param remote: a reference to the L{buildbot.slave.bot.SlaveBuilder}
@@ -692,7 +713,7 @@ class BuilderControl(components.Adapter):
         if not bs.isFinished():
             return
 
-        ss = bs.getSourceStamp()
+        ss = bs.getSourceStamp(absolute=True)
         req = base.BuildRequest(reason, ss, self.original.name)
         self.requestBuild(req)
 
@@ -701,10 +722,7 @@ class BuilderControl(components.Adapter):
         raise NotImplementedError
 
     def getBuild(self, number):
-        for b in self.original.building:
-            if b.build_status.number == number:
-                return b
-        return None
+        return self.original.getBuild(number)
 
     def ping(self, timeout=30):
         if not self.original.slaves:
