@@ -399,6 +399,15 @@ class LogLineObserver(LogObserver):
         self.stderrParser.lineReceived = self.errLineReceived
         self.stderrParser.transport = self
 
+    def setMaxLineLength(self, max_length):
+        """
+        Set the maximum line length: lines longer than max_length are
+        dropped.  Default is 16384 bytes.  Use sys.maxint for effective
+        infinity.
+        """
+        self.stdoutParser.MAX_LENGTH = max_length
+        self.stderrParser.MAX_LENGTH = max_length
+
     def outReceived(self, data):
         self.stdoutParser.dataReceived(data)
 
@@ -622,8 +631,8 @@ class BuildStep:
     def getProperty(self, propname):
         return self.build.getProperty(propname)
 
-    def setProperty(self, propname, value):
-        self.build.setProperty(propname, value)
+    def setProperty(self, propname, value, source="Step"):
+        self.build.setProperty(propname, value, source)
 
     def startStep(self, remote):
         """Begin the step. This returns a Deferred that will fire when the
@@ -1082,3 +1091,8 @@ class LoggingBuildStep(BuildStep):
         self.step_status.setText(self.getText(cmd, results))
         self.step_status.setText2(self.maybeGetText2(cmd, results))
 
+# (WithProeprties used to be available in this module)
+from buildbot.process.properties import WithProperties
+_hush_pyflakes = [WithProperties]
+del _hush_pyflakes
+
