@@ -3,6 +3,7 @@
 # N.B.: don't import anything that might pull in a reactor yet. Some of our
 # subcommands want to load modules that need the gtk reactor.
 import os, sys, stat, re, time
+import traceback
 from twisted.python import usage, util, runtime
 
 # this is mostly just a front-end for mktap, twistd, and kill(1), but in the
@@ -762,6 +763,8 @@ class TryOptions(usage.Options):
 
         ["builder", "b", None,
          "Run the trial build on this Builder. Can be used multiple times."],
+        ["properties", None, None,
+         "A set of properties made available in the build environment, format:prop=value,propb=valueb..."],
         ]
 
     optFlags = [
@@ -771,10 +774,21 @@ class TryOptions(usage.Options):
     def __init__(self):
         super(TryOptions, self).__init__()
         self['builders'] = []
+        self['properties'] = {}
 
     def opt_builder(self, option):
         self['builders'].append(option)
 
+    def opt_properties(self, option):
+        # We need to split the value of this option into a dictionary of properties
+        properties = {}
+        propertylist = option.split(",")
+        for i in range(0,len(propertylist)):
+            print propertylist[i]
+            splitproperty = propertylist[i].split("=")
+            properties[splitproperty[0]] = splitproperty[1]
+        self['properties'] = properties
+
     def opt_patchlevel(self, option):
         self['patchlevel'] = int(option)
 
@@ -809,6 +823,39 @@ def doTryServer(config):
     os.rename(tmpfile, newfile)
 
 
+class CheckConfigOptions(usage.Options):
+    optFlags = [
+        ['quiet', 'q', "Don't display error messages or tracebacks"],
+    ]
+
+    def getSynopsis(self):
+        return "Usage		:buildbot checkconfig [configFile]\n" + \
+         "		If not specified, 'master.cfg' will be used as 'configFile'"
+
+    def parseArgs(self, *args):
+        if len(args) >= 1:
+            self['configFile'] = args[0]
+        else:
+            self['configFile'] = 'master.cfg'
+
+
+def doCheckConfig(config):
+    quiet = config.get('quiet')
+    configFile = config.get('configFile')
+    try:
+        from buildbot.scripts.checkconfig import ConfigLoader
+        ConfigLoader(configFile)
+    except:
+        if not quiet:
+            # Print out the traceback in a nice format
+            t, v, tb = sys.exc_info()
+            traceback.print_exception(t, v, tb)
+        sys.exit(1)
+
+    if not quiet:
+        print "Config file is good!"
+
+
 class Options(usage.Options):
     synopsis = "Usage:    buildbot <command> [command options]"
 
@@ -847,6 +894,9 @@ class Options(usage.Options):
         ['tryserver', None, TryServerOptions,
          "buildmaster-side 'try' support function, not for users"],
 
+        ['checkconfig', None, CheckConfigOptions,
+         "test the validity of a master.cfg config file"],
+
         # TODO: 'watch'
         ]
 
@@ -906,5 +956,7 @@ def run():
         doTry(so)
     elif command == "tryserver":
         doTryServer(so)
+    elif command == "checkconfig":
+        doCheckConfig(so)
 
 
