@@ -110,25 +110,27 @@ class FileUpload(BuildStep):
 
     def start(self):
         version = self.slaveVersion("uploadFile")
+        properties = self.build.getProperties()
+
         if not version:
             m = "slave is too old, does not know about uploadFile"
             raise BuildSlaveTooOldError(m)
 
-        source = self.slavesrc
-        masterdest = self.masterdest
+        source = properties.render(self.slavesrc)
+        masterdest = properties.render(self.masterdest)
         # we rely upon the fact that the buildmaster runs chdir'ed into its
         # basedir to make sure that relative paths in masterdest are expanded
         # properly. TODO: maybe pass the master's basedir all the way down
         # into the BuildStep so we can do this better.
-        target = os.path.expanduser(masterdest)
+        masterdest = os.path.expanduser(masterdest)
         log.msg("FileUpload started, from slave %r to master %r"
-                % (source, target))
+                % (source, masterdest))
 
         self.step_status.setColor('yellow')
         self.step_status.setText(['uploading', os.path.basename(source)])
 
         # we use maxsize to limit the amount of data on both sides
-        fileWriter = _FileWriter(self.masterdest, self.maxsize, self.mode)
+        fileWriter = _FileWriter(masterdest, self.maxsize, self.mode)
 
         # default arguments
         args = {
@@ -236,6 +238,8 @@ class FileDownload(BuildStep):
         self.mode = mode
 
     def start(self):
+        properties = self.build.getProperties()
+
         version = self.slaveVersion("downloadFile")
         if not version:
             m = "slave is too old, does not know about downloadFile"
@@ -243,8 +247,8 @@ class FileDownload(BuildStep):
 
         # we are currently in the buildmaster's basedir, so any non-absolute
         # paths will be interpreted relative to that
-        source = os.path.expanduser(self.mastersrc)
-        slavedest = self.slavedest
+        source = os.path.expanduser(properties.render(self.mastersrc))
+        slavedest = properties.render(self.slavedest)
         log.msg("FileDownload started, from master %r to slave %r" %
                 (source, slavedest))
 
@@ -267,7 +271,7 @@ class FileDownload(BuildStep):
 
         # default arguments
         args = {
-            'slavedest': self.slavedest,
+            'slavedest': slavedest,
             'maxsize': self.maxsize,
             'reader': fileReader,
             'blocksize': self.blocksize,
