@@ -1,15 +1,17 @@
 # -*- test-case-name: buildbot.test.test_status -*-
 
 import email, os
+import operator
 
+from zope.interface import implements
 from twisted.internet import defer, reactor
 from twisted.trial import unittest
 
 from buildbot import interfaces
 from buildbot.sourcestamp import SourceStamp
 from buildbot.process.base import BuildRequest
-from buildbot.twcompat import implements, providedBy, maybeWait
-from buildbot.status import builder, base
+from buildbot.status import builder, base, words
+from buildbot.changes.changes import Change
 
 mail = None
 try:
@@ -109,10 +111,7 @@ class MyBuild(builder.BuildStatus):
         return self.testlogs
 
 class MyLookup:
-    if implements:
-        implements(interfaces.IEmailLookup)
-    else:
-        __implements__ = interfaces.IEmailLookup,
+    implements(interfaces.IEmailLookup)
 
     def getAddress(self, user):
         d = defer.Deferred()
@@ -138,7 +137,11 @@ class Mail(unittest.TestCase):
         return MyBuild(self.builder, number, results)
 
     def failUnlessIn(self, substring, string):
-        self.failUnless(string.find(substring) != -1)
+        self.failUnless(string.find(substring) != -1,
+                        "didn't see '%s' in '%s'" % (substring, string))
+
+    def getProjectName(self):
+        return "PROJECT"
 
     def getBuildbotURL(self):
         return "BUILDBOT_URL"
@@ -165,7 +168,7 @@ class Mail(unittest.TestCase):
         self.failUnlessIn("To: bob@dev.com, recip2@example.com, "
                           "recip@example.com\n", t)
         self.failUnlessIn("From: buildbot@example.com\n", t)
-        self.failUnlessIn("Subject: buildbot success in builder1\n", t)
+        self.failUnlessIn("Subject: buildbot success in PROJECT on builder1\n", t)
         self.failUnlessIn("Date: ", t)
         self.failUnlessIn("Build succeeded!\n", t)
         self.failUnlessIn("Buildbot URL: BUILDBOT_URL\n", t)
@@ -190,7 +193,7 @@ class Mail(unittest.TestCase):
         self.failUnlessIn("To: recip2@example.com, "
                           "recip@example.com\n", t)
         self.failUnlessIn("From: buildbot@example.com\n", t)
-        self.failUnlessIn("Subject: buildbot success in builder1\n", t)
+        self.failUnlessIn("Subject: buildbot success in PROJECT on builder1\n", t)
         self.failUnlessIn("Build succeeded!\n", t)
         self.failUnlessIn("Buildbot URL: BUILDBOT_URL\n", t)
 
@@ -291,7 +294,7 @@ class Mail(unittest.TestCase):
         self.failUnlessIn("To: dev3@dev.com, dev4@dev.com, "
                           "recip2@example.com, recip@example.com\n", t)
         self.failUnlessIn("From: buildbot@example.com\n", t)
-        self.failUnlessIn("Subject: buildbot failure in builder1\n", t)
+        self.failUnlessIn("Subject: buildbot failure in PROJECT on builder1\n", t)
         self.failUnlessIn("The Buildbot has detected a new failure", t)
         self.failUnlessIn("BUILD FAILED: snarkleack polarization failed\n", t)
         self.failUnlessEqual(r, ["dev3@dev.com", "dev4@dev.com",
@@ -317,7 +320,7 @@ class Mail(unittest.TestCase):
         self.failUnless(len(self.messages) == 1)
         m,r = self.messages.pop()
         t = m.as_string()
-        self.failUnlessIn("Subject: buildbot warnings in builder1\n", t)
+        self.failUnlessIn("Subject: buildbot warnings in PROJECT on builder1\n", t)
         m2 = email.message_from_string(t)
         p = m2.get_payload()
         self.failUnlessEqual(len(p), 3)
@@ -360,7 +363,7 @@ class Mail(unittest.TestCase):
         # otherwise a malicious SMTP server could make us consume lots of
         # memory.
         d.addCallback(self.stall, 0.1)
-        return maybeWait(d)
+        return d
 
 if not mail:
     Mail.skip = "the Twisted Mail package is not installed"
@@ -394,7 +397,7 @@ class Results(unittest.TestCase):
         res = b.getTestResults()
         self.failUnlessEqual(res.keys(), [testname])
         t = res[testname]
-        self.failUnless(providedBy(t, interfaces.ITestResult))
+        self.failUnless(interfaces.ITestResult.providedBy(t))
         self.failUnlessEqual(t.getName(), testname)
         self.failUnlessEqual(t.getResults(), builder.SUCCESS)
         self.failUnlessEqual(t.getText(), ["passed"])
@@ -629,7 +632,8 @@ class Log(unittest.TestCase):
         s = MyLogConsumer()
         d = l1.subscribeConsumer(s)
         d.addCallback(self._testConsumer_1, s)
-        return maybeWait(d, 5)
+        return d
+    testConsumer.timeout = 5
     def _testConsumer_1(self, res, s):
         self.failIf(s.chunks)
         self.failUnless(s.finished)
@@ -721,12 +725,13 @@ class Log(unittest.TestCase):
         # NetstringReceiver.dataReceived where it does
         # self.transport.loseConnection() because of the NetstringParseError,
         # however self.transport is None
-        return maybeWait(d, 5)
+        return d
     testLargeSummary.timeout = 5
 
 config_base = """
 from buildbot.process import factory
 from buildbot.steps import dummy
+from buildbot.buildslave import BuildSlave
 s = factory.s
 
 f1 = factory.QuickBuildFactory('fakerep', 'cvsmodule', configure=None)
@@ -737,8 +742,7 @@ f2 = factory.BuildFactory([
     ])
 
 BuildmasterConfig = c = {}
-c['bots'] = [['bot1', 'sekrit']]
-c['sources'] = []
+c['slaves'] = [BuildSlave('bot1', 'sekrit')]
 c['schedulers'] = []
 c['builders'] = []
 c['builders'].append({'name':'quick', 'slavename':'bot1',
@@ -859,7 +863,7 @@ class Subscription(RunMixin, unittest.TestCase):
 
         d = self.connectSlave(builders=["dummy", "testdummy"])
         d.addCallback(self._testSlave_1, t1)
-        return maybeWait(d)
+        return d
 
     def _testSlave_1(self, res, t1):
         self.failUnlessEqual(len(t1.events), 2)
@@ -910,9 +914,11 @@ class Subscription(RunMixin, unittest.TestCase):
         self.failUnless(b)
         self.failUnlessEqual(b.getBuilder().getName(), "dummy")
         self.failUnlessEqual(b.getNumber(), 0)
-        self.failUnlessEqual(b.getSourceStamp(), (None, None, None))
+        self.failUnlessEqual(b.getSourceStamp().branch, None)
+        self.failUnlessEqual(b.getSourceStamp().patch, None)
+        self.failUnlessEqual(b.getSourceStamp().revision, None)
         self.failUnlessEqual(b.getReason(), "forced build for testing")
-        self.failUnlessEqual(b.getChanges(), [])
+        self.failUnlessEqual(b.getChanges(), ())
         self.failUnlessEqual(b.getResponsibleUsers(), [])
         self.failUnless(b.isFinished())
         self.failUnlessEqual(b.getText(), ['build', 'successful'])
@@ -976,3 +982,249 @@ class Client(unittest.TestCase):
         self.failUnless(isinstance(b2, client.RemoteBuilder))
         b3 = client.makeRemote(None)
         self.failUnless(b3 is None)
+
+
+class ContactTester(unittest.TestCase):
+    def test_notify_invalid_syntax(self):
+        irc = MyContact()
+        self.assertRaises(words.UsageError, lambda args, who: irc.command_NOTIFY(args, who), "", "mynick")
+
+    def test_notify_list(self):
+        irc = MyContact()
+        irc.command_NOTIFY("list", "mynick")
+        self.failUnlessEqual(irc.message, "The following events are being notified: []", "empty notify list")
+
+        irc.message = ""
+        irc.command_NOTIFY("on started", "mynick")
+        self.failUnlessEqual(irc.message, "The following events are being notified: ['started']", "on started")
+
+        irc.message = ""
+        irc.command_NOTIFY("on finished", "mynick")
+        self.failUnlessEqual(irc.message, "The following events are being notified: ['started', 'finished']", "on finished")
+
+        irc.message = ""
+        irc.command_NOTIFY("off", "mynick")
+        self.failUnlessEqual(irc.message, "The following events are being notified: []", "off all")
+
+        irc.message = ""
+        irc.command_NOTIFY("on", "mynick")
+        self.failUnlessEqual(irc.message, "The following events are being notified: ['started', 'finished']", "on default set")
+
+        irc.message = ""
+        irc.command_NOTIFY("off started", "mynick")
+        self.failUnlessEqual(irc.message, "The following events are being notified: ['finished']", "off started")
+
+        irc.message = ""
+        irc.command_NOTIFY("on success failed exception", "mynick")
+        self.failUnlessEqual(irc.message, "The following events are being notified: ['failed', 'finished', 'exception', 'success']", "on multiple events")
+
+    def test_notification_default(self):
+        irc = MyContact()
+
+        my_builder = MyBuilder("builder78")
+        my_build = MyIrcBuild(my_builder, 23, builder.SUCCESS)
+
+        irc.buildStarted(my_builder.getName(), my_build)
+        self.failUnlessEqual(irc.message, "", "No notification with default settings")
+
+        irc.buildFinished(my_builder.getName(), my_build, None)
+        self.failUnlessEqual(irc.message, "", "No notification with default settings")
+
+    def test_notification_started(self):
+        irc = MyContact()
+
+        my_builder = MyBuilder("builder78")
+        my_build = MyIrcBuild(my_builder, 23, builder.SUCCESS)
+        my_build.changes = (
+            Change(who = 'author1', files = ['file1'], comments = 'comment1', revision = 123),
+            Change(who = 'author2', files = ['file2'], comments = 'comment2', revision = 456),
+            )
+
+        irc.command_NOTIFY("on started", "mynick")
+
+        irc.message = ""
+        irc.buildStarted(my_builder.getName(), my_build)
+        self.failUnlessEqual(irc.message, "build #23 of builder78 started including [123, 456]", "Start notification generated with notify_events=['started']")
+
+        irc.message = ""
+        irc.buildFinished(my_builder.getName(), my_build, None)
+        self.failUnlessEqual(irc.message, "", "No finished notification with notify_events=['started']")
+
+    def test_notification_finished(self):
+        irc = MyContact()
+
+        my_builder = MyBuilder("builder834")
+        my_build = MyIrcBuild(my_builder, 862, builder.SUCCESS)
+        my_build.changes = (
+            Change(who = 'author1', files = ['file1'], comments = 'comment1', revision = 943),
+            )
+
+        irc.command_NOTIFY("on finished", "mynick")
+
+        irc.message = ""
+        irc.buildStarted(my_builder.getName(), my_build)
+        self.failUnlessEqual(irc.message, "", "No started notification with notify_events=['finished']")
+
+        irc.message = ""
+        irc.buildFinished(my_builder.getName(), my_build, None)
+        self.failUnlessEqual(irc.message, "build #862 of builder834 is complete: Success [step1 step2]  Build details are at http://myserver/mypath?build=765", "Finish notification generated with notify_events=['finished']")
+
+    def test_notification_success(self):
+        irc = MyContact()
+
+        my_builder = MyBuilder("builder834")
+        my_build = MyIrcBuild(my_builder, 862, builder.SUCCESS)
+        my_build.changes = (
+            Change(who = 'author1', files = ['file1'], comments = 'comment1', revision = 943),
+            )
+
+        irc.command_NOTIFY("on success", "mynick")
+
+        irc.message = ""
+        irc.buildStarted(my_builder.getName(), my_build)
+        self.failUnlessEqual(irc.message, "", "No started notification with notify_events=['success']")
+
+        irc.message = ""
+        irc.buildFinished(my_builder.getName(), my_build, None)
+        self.failUnlessEqual(irc.message, "build #862 of builder834 is complete: Success [step1 step2]  Build details are at http://myserver/mypath?build=765", "Finish notification generated on success with notify_events=['success']")
+
+        irc.message = ""
+        my_build.results = builder.FAILURE
+        irc.buildFinished(my_builder.getName(), my_build, None)
+        self.failUnlessEqual(irc.message, "", "No finish notification generated on failure with notify_events=['success']")
+
+        irc.message = ""
+        my_build.results = builder.EXCEPTION
+        irc.buildFinished(my_builder.getName(), my_build, None)
+        self.failUnlessEqual(irc.message, "", "No finish notification generated on exception with notify_events=['success']")
+
+    def test_notification_failed(self):
+        irc = MyContact()
+
+        my_builder = MyBuilder("builder834")
+        my_build = MyIrcBuild(my_builder, 862, builder.FAILURE)
+        my_build.changes = (
+            Change(who = 'author1', files = ['file1'], comments = 'comment1', revision = 943),
+            )
+
+        irc.command_NOTIFY("on failed", "mynick")
+
+        irc.message = ""
+        irc.buildStarted(my_builder.getName(), my_build)
+        self.failUnlessEqual(irc.message, "", "No started notification with notify_events=['failed']")
+
+        irc.message = ""
+        irc.buildFinished(my_builder.getName(), my_build, None)
+        self.failUnlessEqual(irc.message, "build #862 of builder834 is complete: Failure [step1 step2]  Build details are at http://myserver/mypath?build=765", "Finish notification generated on failure with notify_events=['failed']")
+
+        irc.message = ""
+        my_build.results = builder.SUCCESS
+        irc.buildFinished(my_builder.getName(), my_build, None)
+        self.failUnlessEqual(irc.message, "", "No finish notification generated on success with notify_events=['failed']")
+
+        irc.message = ""
+        my_build.results = builder.EXCEPTION
+        irc.buildFinished(my_builder.getName(), my_build, None)
+        self.failUnlessEqual(irc.message, "", "No finish notification generated on exception with notify_events=['failed']")
+
+    def test_notification_exception(self):
+        irc = MyContact()
+
+        my_builder = MyBuilder("builder834")
+        my_build = MyIrcBuild(my_builder, 862, builder.EXCEPTION)
+        my_build.changes = (
+            Change(who = 'author1', files = ['file1'], comments = 'comment1', revision = 943),
+            )
+
+        irc.command_NOTIFY("on exception", "mynick")
+
+        irc.message = ""
+        irc.buildStarted(my_builder.getName(), my_build)
+        self.failUnlessEqual(irc.message, "", "No started notification with notify_events=['exception']")
+
+        irc.message = ""
+        irc.buildFinished(my_builder.getName(), my_build, None)
+        self.failUnlessEqual(irc.message, "build #862 of builder834 is complete: Exception [step1 step2]  Build details are at http://myserver/mypath?build=765", "Finish notification generated on failure with notify_events=['exception']")
+
+        irc.message = ""
+        my_build.results = builder.SUCCESS
+        irc.buildFinished(my_builder.getName(), my_build, None)
+        self.failUnlessEqual(irc.message, "", "No finish notification generated on success with notify_events=['exception']")
+
+        irc.message = ""
+        my_build.results = builder.FAILURE
+        irc.buildFinished(my_builder.getName(), my_build, None)
+        self.failUnlessEqual(irc.message, "", "No finish notification generated on exception with notify_events=['exception']")
+
+class MyIrcBuild(builder.BuildStatus):
+    results = None
+
+    def __init__(self, parent, number, results):
+        builder.BuildStatus.__init__(self, parent, number)
+        self.results = results
+
+    def getResults(self):
+        return self.results
+
+    def getText(self):
+        return ('step1', 'step2')
+
+class URLProducer:
+    def getURLForThing(self, build):
+        return 'http://myserver/mypath?build=765'
+
+class MyChannel:
+    categories = None
+    status = URLProducer()
+
+    def __init__(self):
+        pass
+
+class MyContact(words.Contact):
+    message = ""
+
+    def __init__(self, channel = MyChannel()):
+        words.Contact.__init__(self, channel)
+        self.message = ""
+
+    def subscribe_to_build_events(self):
+        pass
+
+    def unsubscribe_from_build_events(self):
+        pass
+
+    def send(self, msg):
+        self.message += msg
+
+class StepStatistics(unittest.TestCase):
+    def testStepStatistics(self):
+        status = builder.BuildStatus(builder.BuilderStatus("test"), 123)
+        status.addStepWithName('step1')
+        status.addStepWithName('step2')
+        status.addStepWithName('step3')
+        status.addStepWithName('step4')
+
+        steps = status.getSteps()
+        (step1, step2, step3, step4) = steps
+
+        step1.setStatistic('test-prop', 1)
+        step3.setStatistic('test-prop', 2)
+        step4.setStatistic('test-prop', 4)
+
+        step1.setStatistic('other-prop', 27)
+        # Just to have some other properties around
+
+        self.failUnlessEqual(step1.getStatistic('test-prop'), 1,
+            'Retrieve an existing property')
+        self.failUnlessEqual(step1.getStatistic('test-prop', 99), 1,
+            "Don't default an existing property")
+        self.failUnlessEqual(step2.getStatistic('test-prop', 99), 99,
+            'Default a non-existant property')
+
+        self.failUnlessEqual(
+            status.getSummaryStatistic('test-prop', operator.add), 7,
+            'Sum property across the build')
+
+        self.failUnlessEqual(
+            status.getSummaryStatistic('test-prop', operator.add, 13), 20,
+            'Sum property across the build with initial value')
