@@ -24,7 +24,7 @@ from buildbot.process import buildstep, base, factory
 from buildbot.buildslave import BuildSlave
 from buildbot.steps import shell, source, python
 from buildbot.status import builder
-from buildbot.status.builder import SUCCESS, FAILURE
+from buildbot.status.builder import SUCCESS, WARNINGS, FAILURE
 from buildbot.test.runutils import RunMixin, rmtree
 from buildbot.test.runutils import makeBuildStep, StepTester
 from buildbot.slave import commands, registry
@@ -200,19 +200,49 @@ class Steps(unittest.TestCase):
         b = f.newBuild([req])
         #for s in b.steps: print s.name
 
+    def failUnlessClones(self, s1, attrnames):
+        f1 = s1.getStepFactory()
+        f,args = f1
+        s2 = f(**args)
+        for name in attrnames:
+            self.failUnlessEqual(getattr(s1, name), getattr(s2, name))
+
+    def clone(self, s1):
+        f1 = s1.getStepFactory()
+        f,args = f1
+        s2 = f(**args)
+        return s2
+
+    def testClone(self):
+        s1 = shell.ShellCommand(command=["make", "test"],
+                                timeout=1234,
+                                workdir="here",
+                                description="yo",
+                                descriptionDone="yoyo",
+                                env={'key': 'value'},
+                                want_stdout=False,
+                                want_stderr=False,
+                                logfiles={"name": "filename"},
+                               )
+        shellparms = (buildstep.BuildStep.parms +
+                      ("remote_kwargs description descriptionDone "
+                       "command logfiles").split() )
+        self.failUnlessClones(s1, shellparms)
+
+
     # test the various methods available to buildsteps
 
     def test_getProperty(self):
         s = makeBuildStep("test_steps.Steps.test_getProperty")
         bs = s.step_status.getBuild()
 
-        s.setProperty("prop1", "value1")
-        s.setProperty("prop2", "value2")
+        s.setProperty("prop1", "value1", "test")
+        s.setProperty("prop2", "value2", "test")
         self.failUnlessEqual(s.getProperty("prop1"), "value1")
         self.failUnlessEqual(bs.getProperty("prop1"), "value1")
         self.failUnlessEqual(s.getProperty("prop2"), "value2")
         self.failUnlessEqual(bs.getProperty("prop2"), "value2")
-        s.setProperty("prop1", "value1a")
+        s.setProperty("prop1", "value1a", "test")
         self.failUnlessEqual(s.getProperty("prop1"), "value1a")
         self.failUnlessEqual(bs.getProperty("prop1"), "value1a")
 
@@ -492,3 +522,157 @@ buildbot/scripts/imaginary.py:18: 'from buildbot import *' used; unable to detec
         self.failUnless("import*=1" in desc)
         self.failIf("misc" in desc)
 
+
+class OrdinaryCompile(shell.Compile):
+    warningPattern = "ordinary line"
+
+class Warnings(StepTester, unittest.TestCase):
+    def testCompile1(self):
+        self.masterbase = "Warnings.testCompile1"
+        step = self.makeStep(shell.Compile)
+        output = \
+"""Compile started
+normal line
+warning: oh noes!
+ordinary line
+error (but we aren't looking for errors now, are we)
+line 23: warning: we are now on line 23
+ending line
+"""
+        log = step.addLog("stdio")
+        log.addStdout(output)
+        log.finish()
+        step.createSummary(log)
+        self.failUnlessEqual(step.getProperty("warnings-count"), 2)
+        logs = {}
+        for log in step.step_status.getLogs():
+            logs[log.getName()] = log
+        self.failUnless("warnings" in logs)
+        lines = logs["warnings"].readlines()
+        self.failUnlessEqual(len(lines), 2)
+        self.failUnlessEqual(lines[0], "warning: oh noes!\n")
+        self.failUnlessEqual(lines[1],
+                             "line 23: warning: we are now on line 23\n")
+
+        cmd = buildstep.RemoteCommand(None, {})
+        cmd.rc = 0
+        results = step.evaluateCommand(cmd)
+        self.failUnlessEqual(results, WARNINGS)
+
+    def testCompile2(self):
+        self.masterbase = "Warnings.testCompile2"
+        step = self.makeStep(shell.Compile, warningPattern="ordinary line")
+        output = \
+"""Compile started
+normal line
+warning: oh noes!
+ordinary line
+error (but we aren't looking for errors now, are we)
+line 23: warning: we are now on line 23
+ending line
+"""
+        log = step.addLog("stdio")
+        log.addStdout(output)
+        log.finish()
+        step.createSummary(log)
+        self.failUnlessEqual(step.getProperty("warnings-count"), 1)
+        logs = {}
+        for log in step.step_status.getLogs():
+            logs[log.getName()] = log
+        self.failUnless("warnings" in logs)
+        lines = logs["warnings"].readlines()
+        self.failUnlessEqual(len(lines), 1)
+        self.failUnlessEqual(lines[0], "ordinary line\n")
+
+        cmd = buildstep.RemoteCommand(None, {})
+        cmd.rc = 0
+        results = step.evaluateCommand(cmd)
+        self.failUnlessEqual(results, WARNINGS)
+
+    def testCompile3(self):
+        self.masterbase = "Warnings.testCompile3"
+        step = self.makeStep(OrdinaryCompile)
+        output = \
+"""Compile started
+normal line
+warning: oh noes!
+ordinary line
+error (but we aren't looking for errors now, are we)
+line 23: warning: we are now on line 23
+ending line
+"""
+        step.setProperty("warnings-count", 10, "test")
+        log = step.addLog("stdio")
+        log.addStdout(output)
+        log.finish()
+        step.createSummary(log)
+        self.failUnlessEqual(step.getProperty("warnings-count"), 11)
+        logs = {}
+        for log in step.step_status.getLogs():
+            logs[log.getName()] = log
+        self.failUnless("warnings" in logs)
+        lines = logs["warnings"].readlines()
+        self.failUnlessEqual(len(lines), 1)
+        self.failUnlessEqual(lines[0], "ordinary line\n")
+
+        cmd = buildstep.RemoteCommand(None, {})
+        cmd.rc = 0
+        results = step.evaluateCommand(cmd)
+        self.failUnlessEqual(results, WARNINGS)
+
+
+class TreeSize(StepTester, unittest.TestCase):
+    def testTreeSize(self):
+        self.slavebase = "TreeSize.testTreeSize.slave"
+        self.masterbase = "TreeSize.testTreeSize.master"
+
+        sb = self.makeSlaveBuilder()
+        step = self.makeStep(shell.TreeSize)
+        d = self.runStep(step)
+        def _check(results):
+            self.failUnlessEqual(results, SUCCESS)
+            kib = step.getProperty("tree-size-KiB")
+            self.failUnless(isinstance(kib, int))
+            self.failUnless(kib < 100) # should be empty, I get '4'
+            s = step.step_status
+            self.failUnlessEqual(" ".join(s.getText()),
+                                 "treesize %d KiB" % kib)
+        d.addCallback(_check)
+        return d
+
+class PerlModuleTest(StepTester, unittest.TestCase):
+    def testAllTestsPassed(self):
+        self.masterbase = "Warnings.testAllTestsPassed"
+        step = self.makeStep(shell.PerlModuleTest)
+        output = \
+"""ok 1
+ok 2
+All tests successful
+Files=1, Tests=123, other stuff
+"""
+        log = step.addLog("stdio")
+        log.addStdout(output)
+        log.finish()
+        step.evaluateCommand(log)
+        ss = step.step_status
+        self.failUnlessEqual(ss.getStatistic('tests-failed'), 0)
+        self.failUnlessEqual(ss.getStatistic('tests-total'), 123)
+        self.failUnlessEqual(ss.getStatistic('tests-passed'), 123)
+
+    def testFailures(self):
+        self.masterbase = "Warnings.testFailures"
+        step = self.makeStep(shell.PerlModuleTest)
+        output = \
+"""
+ok 1
+ok 2
+3/7 subtests failed
+"""
+        log = step.addLog("stdio")
+        log.addStdout(output)
+        log.finish()
+        step.evaluateCommand(log)
+        ss = step.step_status
+        self.failUnlessEqual(ss.getStatistic('tests-failed'), 3)
+        self.failUnlessEqual(ss.getStatistic('tests-total'), 7)
+        self.failUnlessEqual(ss.getStatistic('tests-passed'), 4)
